@@ -4,7 +4,7 @@
  * Author:      Julian Smart
  * Modified by:
  * Created:     01/02/97
- * RCS-ID:      $Id: chkconf.h,v 1.4 2004/05/23 20:51:02 JS Exp $
+ * RCS-ID:      $Id: chkconf.h,v 1.7 2005/07/27 15:02:04 SC Exp $
  * Copyright:   (c) Julian Smart
  * Licence:     wxWindows licence
  */
@@ -16,13 +16,30 @@
  * disable the settings which don't work for some compilers
  */
 
-/*
- * wxUSE_DEBUG_NEW_ALWAYS doesn't work with CodeWarrior
- */
-
 #if defined(__MWERKS__)
     #undef wxUSE_DEBUG_NEW_ALWAYS
     #define wxUSE_DEBUG_NEW_ALWAYS      0
+
+    /* DS: Fixes compilation when wxUSE_FATAL_EXCEPTION is 1 */
+    #ifndef wxTYPE_SA_HANDLER
+        #define wxTYPE_SA_HANDLER int
+    #endif
+#endif
+
+/*
+ * use OS X CoreGraphics (1) or QuickDraw (0) for rendering
+ */
+
+#ifndef wxMAC_USE_CORE_GRAPHICS
+    #define wxMAC_USE_CORE_GRAPHICS 0
+#endif
+
+/*
+ * native (1) or emulated (0) toolbar
+ */
+
+#ifndef wxMAC_USE_NATIVE_TOOLBAR
+    #define wxMAC_USE_NATIVE_TOOLBAR 0
 #endif
 
 #endif
