@@ -4,7 +4,7 @@
 // Author:      Stefan Csomor
 // Modified by:
 // Created:     1998-01-01
-// RCS-ID:      $Id: dcprint.h,v 1.4 2004/05/23 20:51:02 JS Exp $
+// RCS-ID:      $Id: dcprint.h,v 1.6 2005/02/18 04:43:23 RN Exp $
 // Copyright:   (c) Stefan Csomor
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -24,9 +24,9 @@ class wxNativePrinterDC ;
 class WXDLLEXPORT wxPrinterDC: public wxDC
 {
  public:
+#if wxUSE_PRINTING_ARCHITECTURE
   DECLARE_CLASS(wxPrinterDC)
 
-#if wxUSE_PRINTING_ARCHITECTURE
   // Create a printer DC
   wxPrinterDC(const wxPrintData& printdata );
   ~wxPrinterDC();
@@ -37,7 +37,9 @@ class WXDLLEXPORT wxPrinterDC: public wxDC
     virtual void EndPage(void) ;
     wxPrintData& GetPrintData() { return m_printData; }
     virtual void DoGetSize( int *width, int *height ) const;
-    
+#if wxMAC_USE_CORE_GRAPHICS
+    void MacSetCGContext( void * cg ) ;
+#endif    
  protected:
     wxPrintData   m_printData ;
     wxNativePrinterDC* m_nativePrinterDC ;
