@@ -4,7 +4,7 @@
 // Author:      Stefan Csomor
 // Modified by: Stefan Csomor
 // Created:     1998-01-01
-// RCS-ID:      $Id: setup0.h,v 1.60 2005/03/21 00:51:19 VZ Exp $
+// RCS-ID:      $Id: setup0.h,v 1.68 2005/07/30 00:36:32 VZ Exp $
 // Copyright:   (c) Stefan Csomor
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -14,7 +14,6 @@
 
 
 /* --- start common options --- */
-
 // ----------------------------------------------------------------------------
 // global settings
 // ----------------------------------------------------------------------------
@@ -162,22 +161,6 @@
     #define wxUSE_UNICODE 0
 #endif
 
-// Set wxUSE_UNICODE_MSLU to 1 if you want to compile wxWidgets in Unicode mode
-// and be able to run compiled apps under Windows 9x as well as NT/2000/XP.
-// This setting enables use of unicows.dll from MSLU (MS Layer for Unicode, see
-// http://www.microsoft.com/globaldev/handson/dev/mslu_announce.mspx). Note
-// that you will have to modify the makefiles to include unicows.lib import
-// library as the first library (see installation instructions in install.txt
-// to learn how to do it when building the library or samples).
-//
-// If your compiler doesn't have unicows.lib, you can get a version of it at
-// http://libunicows.sourceforge.net
-//
-// Default is 0
-//
-// Recommended setting: 0 (1 if you want to deploy Unicode apps on 9x systems)
-#define wxUSE_UNICODE_MSLU 0
-
 // Setting wxUSE_WCHAR_T to 1 gives you some degree of Unicode support without
 // compiling the program in Unicode mode. More precisely, it will be possible
 // to construct wxString from a wide (Unicode) string and convert any wxString
@@ -267,9 +250,12 @@
 // If enabled (1), compiles wxWidgets streams classes
 #define wxUSE_STREAMS       1
 
-// Use standard C++ streams if 1. If 0, use wxWin streams implementation.
+// Use standard C++ streams if 1. If 0, use wxWin streams implementation only.
 #define wxUSE_STD_IOSTREAM  0
 
+// Enable conversion to standard C++ string if 1.
+#define wxUSE_STD_STRING  0
+
 // ----------------------------------------------------------------------------
 // non GUI features selection
 // ----------------------------------------------------------------------------
@@ -487,20 +473,6 @@
 // Recommended setting: 1 
 #define wxUSE_MEDIACTRL     1
 
-// Use QuickTime
-//
-// Default is 0
-//
-// Recommended setting: 1 if you have the QT SDK installed and you need it, else 0
-#define wxUSE_QUICKTIME     0
-
-// Use DirectShow (requires linkage to strmiids.lib)
-//
-// Default is 0
-//
-// Recommended setting: 1 if the DirectX 7 SDK is installed (highly recommended), else 0
-#define wxUSE_DIRECTSHOW    0
-
 // Use GStreamer for Unix (req a lot of dependancies)
 //
 // Default is 0
@@ -981,7 +953,7 @@
 #define wxODBC_FWD_ONLY_CURSORS	 1
 
 // Default is 0.  Set to 1 to use the deprecated classes, enum types, function,
-// member variables.  With a setting of 1, full backward compatability with the
+// member variables.  With a setting of 1, full backward compatibility with the
 // 2.0.x release is possible. It is STRONGLY recommended that this be set to 0,
 // as future development will be done only on the non-deprecated
 // functions/classes/member variables/etc.
@@ -1063,15 +1035,14 @@
 // override some settings for Metrowerks
 //
 // VZ: isn't this file only used when building with Metrowerks anyhow?
+// CS: no, it is also used by the XCode projects 
 #ifdef __MWERKS__
     #undef wxUSE_DEBUG_CONTEXT
     #define wxUSE_DEBUG_CONTEXT 1
 
     #undef wxUSE_STD_IOSTREAM
-    #define wxUSE_STD_IOSTREAM  1
-
-    #undef wxUSE_STACKWALKER
-    #define wxUSE_STACKWALKER 0
+    // CS: I have to set this to 0 now, as shared builds are having problems 
+    #define wxUSE_STD_IOSTREAM  0
 #endif
 
 // things not implemented under Mac
@@ -1084,6 +1055,9 @@
 #undef wxUSE_TOGGLEBTN
 #define wxUSE_TOGGLEBTN 0
 
+#undef wxUSE_STACKWALKER
+#define wxUSE_STACKWALKER 0
+
 // wxWebKit is a wrapper for Apple's WebKit framework, use it if you want to
 // embed the Safari browser control
 // 0 by default because of Jaguar compatibility problems
@@ -1093,6 +1067,11 @@
 // Set to 0 for no libmspack
 #define wxUSE_LIBMSPACK     0
 
+// native toolbar does not support embedding controls yet, please test by setting to 1
+#define wxMAC_USE_NATIVE_TOOLBAR 0
+
+
+
 
 #endif
     // _WX_SETUP_H_
