@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: fontenum.h,v 1.5 2004/08/25 12:17:51 ABX Exp $
+// RCS-ID:      $Id: fontenum.h,v 1.6 2004/10/13 14:04:19 ABX Exp $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -24,7 +24,7 @@ class wxFontEnumerator: public wxObject
 {
 DECLARE_CLASS(wxFontEnumerator)
 public:
-    wxFontEnumerator() {};
+    wxFontEnumerator() {}
 
     // Enumerate the fonts.
     bool Enumerate();
