@@ -2,7 +2,7 @@
 // Name:        choice.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: choice.cpp,v 1.77 2004/06/24 20:04:01 RD Exp $
+// Id:          $Id: choice.cpp,v 1.85 2005/07/22 18:01:44 ABX Exp $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -38,6 +38,7 @@ extern bool   g_blockEventsOnDrag;
 // "activate"
 //-----------------------------------------------------------------------------
 
+extern "C" {
 static void gtk_choice_clicked_callback( GtkWidget *WXUNUSED(widget), wxChoice *choice )
 {
     if (g_isIdle)
@@ -47,6 +48,29 @@ static void gtk_choice_clicked_callback( GtkWidget *WXUNUSED(widget), wxChoice *
 
     if (g_blockEventsOnDrag) return;
 
+    int selection = wxNOT_FOUND;
+
+#ifdef __WXGTK20__
+    selection = gtk_option_menu_get_history( GTK_OPTION_MENU(choice->GetHandle()) );
+#else
+    GtkMenuShell *menu_shell = GTK_MENU_SHELL( gtk_option_menu_get_menu( GTK_OPTION_MENU(choice->GetHandle()) ) );
+    int count = 0;
+
+    GList *child = menu_shell->children;
+    while (child)
+    {
+        GtkBin *bin = GTK_BIN( child->data );
+        if (!bin->child)
+        {
+            selection = count;
+            break;
+        }
+        child = child->next;
+        count++;
+    }
+#endif
+    choice->m_selection_hack = selection;
+
     wxCommandEvent event(wxEVT_COMMAND_CHOICE_SELECTED, choice->GetId() );
     int n = choice->GetSelection();
 
@@ -61,6 +85,7 @@ static void gtk_choice_clicked_callback( GtkWidget *WXUNUSED(widget), wxChoice *
 
     choice->GetEventHandler()->ProcessEvent(event);
 }
+}
 
 //-----------------------------------------------------------------------------
 // wxChoice
@@ -90,16 +115,16 @@ bool wxChoice::Create( wxWindow *parent, wxWindowID id,
                        int n, const wxString choices[],
                        long style, const wxValidator& validator, const wxString &name )
 {
-    m_needParent = TRUE;
+    m_needParent = true;
 #if (GTK_MINOR_VERSION > 0)
-    m_acceptsFocus = TRUE;
+    m_acceptsFocus = true;
 #endif
 
     if (!PreCreation( parent, pos, size ) ||
         !CreateBase( parent, id, pos, size, style, validator, name ))
     {
         wxFAIL_MSG( wxT("wxChoice creation failed") );
-        return FALSE;
+        return false;
     }
 
     m_widget = gtk_option_menu_new();
@@ -111,6 +136,9 @@ bool wxChoice::Create( wxWindow *parent, wxWindowID id,
         m_strings = new wxSortedArrayString;
     }
 
+    // begin with no selection
+    m_selection_hack = wxNOT_FOUND;
+
     GtkWidget *menu = gtk_menu_new();
 
     for (int i = 0; i < n; i++)
@@ -125,7 +153,7 @@ bool wxChoice::Create( wxWindow *parent, wxWindowID id,
     PostCreation(size);
     SetBestSize(size); // need this too because this is a wxControlWithItems
 
-    return TRUE;
+    return true;
 }
 
 wxChoice::~wxChoice()
@@ -154,6 +182,13 @@ int wxChoice::DoInsert( const wxString &item, int pos )
 
     GtkWidget *menu = gtk_option_menu_get_menu( GTK_OPTION_MENU(m_widget) );
 
+    // if the item to insert is at or before the selection, and the selection is valid
+    if ((pos <= m_selection_hack) && (m_selection_hack != wxNOT_FOUND))
+    {
+        // move the selection forward one
+        m_selection_hack++;
+    }
+
     return GtkAddHelper(menu, pos, item);
 }
 
@@ -212,7 +247,7 @@ void wxChoice::Clear()
     {
         // destroy the data (due to Robert's idea of using wxList<wxObject>
         // and not wxList<wxClientData> we can't just say
-        // m_clientList.DeleteContents(TRUE) - this would crash!
+        // m_clientList.DeleteContents(true) - this would crash!
         wxList::compatibility_iterator node = m_clientList.GetFirst();
         while ( node )
         {
@@ -224,6 +259,9 @@ void wxChoice::Clear()
 
     if ( m_strings )
         m_strings->Clear();
+
+    // begin with no selection
+    m_selection_hack = wxNOT_FOUND;
 }
 
 void wxChoice::Delete( int n )
@@ -237,6 +275,18 @@ void wxChoice::Delete( int n )
 
     wxCHECK_RET( n >= 0 && n < count, _T("invalid index in wxChoice::Delete") );
 
+    // if the item to delete is before the selection, and the selection is valid
+    if ((n < m_selection_hack) && (m_selection_hack != wxNOT_FOUND))
+    {
+        // move the selection back one
+        m_selection_hack--;
+    }
+    else if (n == m_selection_hack)
+    {
+        // invalidate the selection
+        m_selection_hack = wxNOT_FOUND;
+    }
+
     const bool hasClientData = m_clientDataItemsType != wxClientData_None;
     const bool hasObjectData = m_clientDataItemsType == wxClientData_Object;
 
@@ -322,32 +372,15 @@ int wxChoice::FindString( const wxString &string ) const
         count++;
     }
 
-    return -1;
+    return wxNOT_FOUND;
 }
 
 int wxChoice::GetSelection() const
 {
     wxCHECK_MSG( m_widget != NULL, -1, wxT("invalid choice") );
 
-#ifdef __WXGTK20__
+    return m_selection_hack;
 
-    return gtk_option_menu_get_history( GTK_OPTION_MENU(m_widget) );
-
-#else
-    GtkMenuShell *menu_shell = GTK_MENU_SHELL( gtk_option_menu_get_menu( GTK_OPTION_MENU(m_widget) ) );
-    int count = 0;
-
-    GList *child = menu_shell->children;
-    while (child)
-    {
-        GtkBin *bin = GTK_BIN( child->data );
-        if (!bin->child) return count;
-        child = child->next;
-        count++;
-    }
-
-    return -1;
-#endif
 }
 
 void wxChoice::SetString( int n, const wxString& str )
@@ -370,8 +403,8 @@ void wxChoice::SetString( int n, const wxString& str )
 
             wxASSERT_MSG( label != NULL , wxT("wxChoice: invalid label") );
 
-            gtk_label_set_text( label, wxGTK_CONV( str ) ); 
-            
+            gtk_label_set_text( label, wxGTK_CONV( str ) );
+
             return;
         }
         child = child->next;
@@ -381,7 +414,7 @@ void wxChoice::SetString( int n, const wxString& str )
 
 wxString wxChoice::GetString( int n ) const
 {
-    wxCHECK_MSG( m_widget != NULL, wxT(""), wxT("invalid choice") );
+    wxCHECK_MSG( m_widget != NULL, wxEmptyString, wxT("invalid choice") );
 
     GtkMenuShell *menu_shell = GTK_MENU_SHELL( gtk_option_menu_get_menu( GTK_OPTION_MENU(m_widget) ) );
     int count = 0;
@@ -411,7 +444,7 @@ wxString wxChoice::GetString( int n ) const
 
     wxFAIL_MSG( wxT("wxChoice: invalid index in GetString()") );
 
-    return wxT("");
+    return wxEmptyString;
 }
 
 int wxChoice::GetCount() const
@@ -435,6 +468,24 @@ void wxChoice::SetSelection( int n )
 
     int tmp = n;
     gtk_option_menu_set_history( GTK_OPTION_MENU(m_widget), (gint)tmp );
+
+    // set the local selection variable manually
+    if ((n >= 0) && (n < GetCount()))
+    {
+        // a valid selection has been made
+        m_selection_hack = n;
+    }
+    else if ((n == wxNOT_FOUND) || (GetCount() == 0))
+    {
+        // invalidates the selection if there are no items
+        // or if it is specifically set to wxNOT_FOUND
+        m_selection_hack = wxNOT_FOUND;
+    }
+    else
+    {
+        // this selects the first item by default if the selection is out of bounds
+        m_selection_hack = 0;
+    }
 }
 
 void wxChoice::DoApplyWidgetStyle(GtkRcStyle *style)
@@ -494,9 +545,9 @@ int wxChoice::GtkAddHelper(GtkWidget *menu, int pos, const wxString& item)
         // normal control, just append
         if (pos == (int)m_clientList.GetCount())
         {
-        gtk_menu_append( GTK_MENU(menu), menu_item );
-        m_clientList.Append( (wxObject*) NULL );
-        index = m_clientList.GetCount() - 1;
+            gtk_menu_append( GTK_MENU(menu), menu_item );
+            m_clientList.Append( (wxObject*) NULL );
+            index = m_clientList.GetCount() - 1;
         }
         else
         {
@@ -514,7 +565,13 @@ int wxChoice::GtkAddHelper(GtkWidget *menu, int pos, const wxString& item)
         ApplyWidgetStyle();
     }
 
-    gtk_signal_connect( GTK_OBJECT( menu_item ), "activate",
+    // The best size of a wxChoice should probably
+    // be changed everytime the control has been
+    // changed, but at least after adding an item
+    // it has to change. Adapted from Matt Ownby.
+    InvalidateBestSize();
+
+    gtk_signal_connect_after( GTK_OBJECT( menu_item ), "activate",
       GTK_SIGNAL_FUNC(gtk_choice_clicked_callback), (gpointer*)this );
 
     gtk_widget_show( menu_item );
@@ -586,4 +643,3 @@ wxChoice::GetClassDefaultAttributes(wxWindowVariant WXUNUSED(variant))
 
 
 #endif // wxUSE_CHOICE
-
