@@ -4,19 +4,19 @@
 // Author:      Stefan Csomor
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: uma.cpp,v 1.59 2004/06/26 10:02:36 SC Exp $
+// RCS-ID:      $Id: uma.cpp,v 1.69.2.2 2006/03/23 16:14:24 JS Exp $
 // Copyright:   (c) Stefan Csomor
 // Licence:     The wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
-#include "wx/defs.h"
+#include "wx/wxprec.h"
 
 #if wxUSE_GUI
 
 #include "wx/dc.h"
-#include <MacTextEditor.h>
 
 #ifndef __DARWIN__
+#  include <MacTextEditor.h>
 #  include <Navigation.h>
 #  if defined(TARGET_CARBON)
 #    if PM_USE_SESSION_APIS
@@ -33,6 +33,9 @@
 #endif
 #include "wx/mac/uma.h"
 
+#if TARGET_API_MAC_OSX
+#include "wx/toplevel.h"
+#endif
 
 // since we have decided that we only support 8.6 upwards we are
 // checking for these minimum requirements in the startup code of
@@ -194,7 +197,7 @@ long UMAGetProcessMode()
     ProcessInfoRec processinfo;
     ProcessSerialNumber procno ;
 
-    procno.highLongOfPSN = NULL ;
+    procno.highLongOfPSN = 0 ;
     procno.lowLongOfPSN = kCurrentProcess ;
     processinfo.processInfoLength = sizeof(ProcessInfoRec);
     processinfo.processName = NULL;
@@ -379,6 +382,14 @@ void UMASetMenuItemShortcut( MenuRef menu , MenuItemIndex item , wxAcceleratorEn
                     macKey = kDownArrowCharCode ;
                     glyph = kMenuDownArrowGlyph ;
                     break ;
+                case WXK_HOME :
+                    macKey = kHomeCharCode ;
+                    glyph = kMenuNorthwestArrowGlyph ;
+                    break ;
+                case WXK_END :
+                    macKey = kEndCharCode ;
+                    glyph = kMenuSoutheastArrowGlyph ;
+                    break ;
                 default :
                     macKey = toupper( key ) ;
                     break ;
@@ -391,7 +402,7 @@ void UMASetMenuItemShortcut( MenuRef menu , MenuItemIndex item , wxAcceleratorEn
 
         // 1d and 1e have special meaning to SetItemCmd, so
         // do not use for these character codes.
-        if (key != WXK_UP && key != WXK_RIGHT)
+        if (key != WXK_UP && key != WXK_RIGHT && key != WXK_DOWN && key != WXK_LEFT)
             SetItemCmd( menu, item , macKey );
 
         SetMenuItemModifiers(menu, item , modifiers ) ;
@@ -670,7 +681,7 @@ void UMAHighlightAndActivateWindow( WindowRef inWindowRef , bool inActivate )
     if ( inWindowRef )
     {
 //        bool isHighlighted = IsWindowHighlited( inWindowRef ) ;
-//        if ( inActivate != isHightlited )
+//        if ( inActivate != isHighlighted )
         GrafPtr port ;
         GetPort( &port ) ;
         SetPortWindowPort( inWindowRef ) ;
@@ -790,21 +801,17 @@ OSStatus UMAPutScrap( Size size , OSType type , void *data )
 
 Rect* UMAGetControlBoundsInWindowCoords(ControlRef theControl, Rect *bounds)
 {
-    wxWindow* win = wxFindControlFromMacControl( theControl ) ;
-    
     GetControlBounds( theControl , bounds ) ;
 #if TARGET_API_MAC_OSX
-    if ( win != NULL && win->MacGetTopLevelWindow() != NULL )   
+    WindowRef tlwref = GetControlOwner( theControl ) ;
+
+    wxTopLevelWindowMac* tlwwx = wxFindWinFromMacWindow( tlwref ) ;
+    if ( tlwwx != NULL && tlwwx->MacUsesCompositing() )
     {
-        int x , y ;
-        x = 0 ;
-        y = 0 ;
-        
-        win->GetParent()->MacWindowToRootWindow( &x , & y ) ;
-        bounds->left += x ;
-        bounds->right += x ;
-        bounds->top += y ;
-        bounds->bottom += y ;
+        ControlRef rootControl = tlwwx->GetPeer()->GetControlRef() ;
+        HIPoint hiPoint = CGPointMake(  0 , 0 ) ;
+        HIViewConvertPoint( &hiPoint , HIViewGetSuperview(theControl) , rootControl  ) ;
+        OffsetRect( bounds , (short) hiPoint.x , (short) hiPoint.y ) ;
     }
 #endif
     return bounds ;
