@@ -566,7 +566,7 @@ public:
         {
 			// cdecl call
             // sometimes the pDispParams does not match the param info for a activex control
-            int nArg = min(func.params.size(), pDispParams->cArgs);
+            int nArg = wxMin(func.params.size(), pDispParams->cArgs);
             for (int i = nArg - 1; i >= 0; i--)
             {
                 VARIANTARG& va = pDispParams->rgvarg[i];
@@ -581,7 +581,7 @@ public:
 
 		if (func.hasOut)
 		{
-            int nArg = min(func.params.size(), pDispParams->cArgs);
+            int nArg = wxMin(func.params.size(), pDispParams->cArgs);
     		m_activeX->GetParent()->ProcessEvent(event);
             for (int i = 0; i < nArg; i++)
             {
