@@ -5,19 +5,19 @@
 # Author:       Peter Yared, Morgan Hua
 #
 # Created:      5/15/03
-# CVS-ID:       $Id: PyDocViewDemo.py,v 1.4 2005/05/24 20:54:47 RD Exp $
+# CVS-ID:       $Id: PyDocViewDemo.py,v 1.6.2.1 2005/12/20 02:12:00 RD Exp $
 # Copyright:    (c) 2003-2005 ActiveGrid, Inc.
 # License:      wxWindows License
 #----------------------------------------------------------------------------
 
 
 import sys
+import os.path
 import wx
 import wx.lib.docview as docview
 import wx.lib.pydocview as pydocview
 import TextEditor
 import FindService
-import os.path
 _ = wx.GetTranslation
 
 
@@ -27,14 +27,15 @@ _ = wx.GetTranslation
 
 class TextEditorApplication(pydocview.DocApp):
 
-
+    SPLASH = "splash.png"
+    
     def OnInit(self):
         # Call the super - this is important!!!
         pydocview.DocApp.OnInit(self)
 
         # Show the splash dialog while everything is loading up
-        if os.path.exists("splash.png"):
-            self.ShowSplash("splash.png")
+        if os.path.exists(TextEditorApplication.SPLASH):
+            self.ShowSplash(TextEditorApplication.SPLASH)
 
         # Set the name and the icon
         self.SetAppName(_("wxPython PyDocView Demo"))
@@ -63,8 +64,8 @@ class TextEditorApplication(pydocview.DocApp):
         optionsService        = self.InstallService(pydocview.DocOptionsService(supportedModes=wx.lib.docview.DOC_MDI))
         windowMenuService     = self.InstallService(pydocview.WindowMenuService())
         filePropertiesService = self.InstallService(pydocview.FilePropertiesService())
-        if os.path.exists("splash.jpg"):
-            aboutService      = self.InstallService(pydocview.AboutService(image=wx.Image("splash.jpg")))
+        if os.path.exists(TextEditorApplication.SPLASH):
+            aboutService      = self.InstallService(pydocview.AboutService(image=wx.Image(TextEditorApplication.SPLASH)))
         else:
             aboutService      = self.InstallService(pydocview.AboutService())
             
@@ -82,13 +83,15 @@ class TextEditorApplication(pydocview.DocApp):
             textTemplate.CreateDocument('', docview.DOC_NEW).OnNewDocument()
 
         # Close the splash dialog
-        if os.path.exists("splash.jpg"):
+        if os.path.exists(TextEditorApplication.SPLASH):
             self.CloseSplash()
         
         # Show the tips dialog
         if os.path.exists("tips.txt"):
             wx.CallAfter(self.ShowTip, wx.GetApp().GetTopWindow(), wx.CreateFileTipProvider("tips.txt", 0))
 
+        wx.UpdateUIEvent.SetUpdateInterval(1000)  # Overhead of updating menus was too much.  Change to update every N milliseconds.
+
         # Tell the framework that everything is great
         return True
 
